-- random walk
require 'shortcuts'

local effect	= gr.newPixelEffect
		[[
			extern number dia;//Tile size
			extern number pwr;
			extern vec2 pos;//Player position
			vec4 effect(vec4 colour,Image img,vec2 percent,vec2 pixel)
			{
			return Texel(img,percent)*(dia/length(pos-pixel)*pwr);
			}
		]]

function love.load()
	gr.setBackgroundColor( 40, 25,  10, 255 )	-- dark brown
	tree		= gr.newImage( 'tree.png' )
	bush		= gr.newImage( 'bush.png' )
	bee = {
		image	= gr.newImage( 'bee2.png' ),
		size	= 0.3,
		loc		= { 20, 8, 0 },		-- x, y, and bee orientation
		buzz	= audio.newSource( 'buzz.ogg'),
		change	= 1					-- bee size increasing(+) or decreasing(-)
		}
	bee.width	= bee.image:getWidth()
	bee.height	= bee.image:getHeight()
	height		= gr.getHeight()	-- "Oh so sad!" - Love's inverted coordinate system
	xnum		= 40
	ynum		= 30
	types		= 15				-- smaller number, more trees(only 2 types used)
	fld			= create_field( xnum, ynum, types )
	sqr_size	= 20				-- size* num = screen size (800x600)
	light		= {bee.loc[1],bee.loc[2]}
	time		= 0
	lumens		= 0.5
	sizeSpot	= 100
	effect:send( "dia", sizeSpot )
	effect:send( "pwr", lumens )
	audio.setVolume( 0.5 )
	audio.play( bee.buzz )
	end

function love.update( dt )
	time = time + dt
	if time > 0.09 then		-- slow down bee
		bee.loc=fly(bee.loc, types)
		time	= 0
		light = {sqr_size*bee.loc[1],height-sqr_size*bee.loc[2]}	-- invert 'y' for Love
		effect:send( "pos", light )
		if os.clock() > 72.5 then audio.stop()	end
		end
	if down('s') then sizeSpot= sizeSpot+1 effect:send("dia",sizeSpot) end
	if down('a') then sizeSpot= sizeSpot-1 effect:send("dia",sizeSpot) end
	if bee.buzz:isStopped( ) then love.event.quit() end
	bee.size = bee.size + bee.change * dt/2
	if bee.size > 0.8 or bee.size <= 0.1 then bee.change = -bee.change end
	gr.setCaption( 'Busy Bee  - fps: '..love.timer.getFPS()..'  time: '..floor(os.clock())..' out of 72 seconds' )
	end

function love.draw()
	gr.setPixelEffect( effect )
	draw_field( fld )
	gr.setPixelEffect( )
	gr.draw(
		bee.image,
		sqr_size*(bee.loc[1]+.5),
		sqr_size*(bee.loc[2]+.5),
		bee.loc[3]*pi/4,
		bee.size, bee.size,
		bee.width/2,
		bee.height/2
		)
	end

function love.keypressed(key)
	if key == 'escape' then love.event.quit() end
	end

function create_field( x, y, s )
	s = s or 3
	local field = {}
	for j = 1, y do
		field[j] = {}
		for i = 1, x do
			field[j][i] = math.random( 0, s )
			end
		end
	return field
	end

function draw_field( grid )
	gr.setColor( 255, 255, 55 )
	for j = 1, #grid do
		for i = 1, #grid[j] do
			local num = grid[j][i]
			if num == types   then gr.draw(tree,i*sqr_size,j*sqr_size,0,.3,.3) end
			if num == types-1 then gr.draw(bush,i*sqr_size,j*sqr_size,0,.1,.1) end
			end
		end
	end

function fly( f, s )	-- f = field & s = number of object types (only 2 used)
	local cell,ix={},0
	-- orthogonal moves
	if f[1] > 1			then ix=ix+1 cell[ix]={f[1]-1,f[2],6} end
	if f[2] > 1			then ix=ix+1 cell[ix]={f[1],f[2]-1,0} end
	if f[1] < #fld[1]-1	then ix=ix+1 cell[ix]={f[1]+1,f[2],2} end
	if f[2] < #fld-1	then ix=ix+1 cell[ix]={f[1],f[2]+1,4} end
	-- diagonal moves
	if f[1] > 1 and f[2] > 1		then ix=ix+1 cell[ix]={f[1]-1,f[2]-1,1,7} end
	if f[1] < #fld[1] and f[2] > 1	then ix=ix+1 cell[ix]={f[1]+1,f[2]-1,1} end
	if f[1] > 1 and f[2] < #fld		then ix=ix+1 cell[ix]={f[1]-1,f[2]+1,5} end
	if f[1] < #fld[1] and f[2] < #fld	then ix=ix+1 cell[ix]={f[1]+1,f[2]+1,3} end

	cell = shuffle( cell )	-- randomize flight direction
	for i = 1, #cell do		-- use first direction only
		if fld[cell[i][2]][cell[i][1]] < s - 1 then
			return cell[i]	-- always one cell available (had to come from some where)
			end
		end
	end

function shuffle( tab )
	local n = #tab
	while n >= 2 do
		local k = math.random( n )
		tab[n], tab[k] = tab[k], tab[n]
		n = n - 1
		end
	return tab
	end
